@@ -5,21 +5,30 @@ from contextlib import ExitStack
 from dataclasses import dataclass, field
 from datetime import datetime, timedelta
 from enum import Enum
-from typing import Any, Callable, Generic, Iterator, Literal, TypeVar
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Generic,
+    Iterator,
+    Literal,
+    Optional,
+    TypeVar,
+)
 
 import grpc
 import isolate_proto
-from isolate.connections.common import is_agent
-from isolate.logs import Log
 from isolate.server.interface import from_grpc, to_serialized_object, to_struct
-from isolate_proto.configuration import GRPC_OPTIONS
 
 from fal import flags
 from fal._serialization import patch_pickle
-from fal.auth import USER, key_credentials
+from fal.auth import UserAccess, key_credentials
 from fal.logging import get_logger
 from fal.logging.trace import TraceContextInterceptor
 
+if TYPE_CHECKING:
+    from isolate.logs import Log
+
 ResultT = TypeVar("ResultT")
 InputT = TypeVar("InputT")
 UNSET = object()
@@ -28,6 +37,7 @@ _DEFAULT_SERIALIZATION_METHOD = "cloudpickle"
 FAL_SERVERLESS_DEFAULT_KEEP_ALIVE = 10
 FAL_SERVERLESS_DEFAULT_MAX_MULTIPLEXING = 1
 FAL_SERVERLESS_DEFAULT_MIN_CONCURRENCY = 0
+FAL_SERVERLESS_DEFAULT_CONCURRENCY_BUFFER = 0
 ALIAS_AUTH_MODES = ["public", "private", "shared"]
 
 logger = get_logger(__name__)
@@ -35,6 +45,9 @@ logger = get_logger(__name__)
 patch_pickle()
 
 
+AuthMode = Optional[Literal["public", "private", "shared"]]
+
+
 class ServerCredentials:
     def to_grpc(self) -> grpc.ChannelCredentials:
         raise NotImplementedError
@@ -43,6 +56,8 @@ class ServerCredentials:
     def base_options(self) -> dict[str, str | int]:
         import json
 
+        from isolate_proto.configuration import GRPC_OPTIONS
+
         grpc_ops: dict[str, str | int] = dict(GRPC_OPTIONS)
         grpc_ops["grpc.enable_retries"] = 1
         grpc_ops["grpc.service_config"] = json.dumps(
@@ -126,17 +141,34 @@ class FalServerlessKeyCredentials(Credentials):
 
 @dataclass
 class AuthenticatedCredentials(Credentials):
-    user = USER
+    user: UserAccess = field(default_factory=UserAccess)
+    team: str | None = None
 
     def to_grpc(self) -> grpc.ChannelCredentials:
-        return grpc.composite_channel_credentials(
+        creds = [
             self.server_credentials.to_grpc(),
-            grpc.access_token_call_credentials(USER.access_token),
-        )
+            grpc.access_token_call_credentials(self.user.access_token),
+        ]
+
+        if self.team:
+            team_id = self.user.get_account(self.team)["user_id"]
+            creds.append(
+                grpc.metadata_call_credentials(_GRPCMetadata("fal-user-id", team_id))
+            )
+
+        return grpc.composite_channel_credentials(*creds)
 
     def to_headers(self) -> dict[str, str]:
-        token = USER.bearer_token
-        return {"Authorization": token}
+        token = self.user.bearer_token
+        headers = {
+            "Authorization": token,
+        }
+
+        if self.team:
+            team_id = self.user.get_account(self.team)["user_id"]
+            headers["X-Fal-User-Id"] = team_id
+
+        return headers
 
 
 @dataclass
@@ -149,6 +181,8 @@ def get_agent_credentials(original_credentials: Credentials) -> Credentials:
     """If running inside a fal Serverless box, use the preconfigured credentials
     instead of the user provided ones."""
 
+    from isolate.connections.common import is_agent
+
     key_creds = key_credentials()
     if is_agent() and key_creds:
         return FalServerlessKeyCredentials(key_creds[0], key_creds[1])
@@ -156,16 +190,22 @@ def get_agent_credentials(original_credentials: Credentials) -> Credentials:
         return original_credentials
 
 
-def get_default_credentials() -> Credentials:
+def get_default_credentials(team: str | None = None) -> Credentials:
+    from fal.config import Config
+
     if flags.AUTH_DISABLED:
         return Credentials()
 
     key_creds = key_credentials()
     if key_creds:
         logger.debug("Using key credentials")
+        if team:
+            raise ValueError("Using explicit team with key credentials is not allowed")
         return FalServerlessKeyCredentials(key_creds[0], key_creds[1])
     else:
-        return AuthenticatedCredentials()
+        config = Config()
+        team = team or config.get_internal("team")
+        return AuthenticatedCredentials(team=team)
 
 
 @dataclass
@@ -196,6 +236,12 @@ class ApplicationInfo:
     max_multiplexing: int
     active_runners: int
     min_concurrency: int
+    concurrency_buffer: int
+    machine_types: list[str]
+    request_timeout: int
+    startup_timeout: int
+    valid_regions: list[str]
+    created_at: datetime
 
 
 @dataclass
@@ -208,14 +254,22 @@ class AliasInfo:
     max_multiplexing: int
     active_runners: int
     min_concurrency: int
+    concurrency_buffer: int
+    machine_types: list[str]
+    request_timeout: int
+    startup_timeout: int
+    valid_regions: list[str]
 
 
 @dataclass
 class RunnerInfo:
     runner_id: str
     in_flight_requests: int
-    expiration_countdown: int
+    expiration_countdown: Optional[int]
     uptime: timedelta
+    external_metadata: dict[str, Any]
+    revision: str
+    alias: str
 
 
 @dataclass
@@ -224,6 +278,7 @@ class HostedRunResult(Generic[ResultT]):
     status: HostedRunStatus
     logs: list[Log] = field(default_factory=list)
     result: ResultT | None = None
+    stream: Any = None
 
 
 @dataclass
@@ -274,9 +329,36 @@ class KeyScope(enum.Enum):
             raise ValueError(f"Unknown KeyScope: {proto}")
 
 
+class DeploymentStrategy(enum.Enum):
+    RECREATE = "recreate"
+    ROLLING = "rolling"
+
+    @staticmethod
+    def from_proto(
+        proto: isolate_proto.DeploymentStrategy.ValueType | None,
+    ) -> DeploymentStrategy:
+        if proto is None:
+            return DeploymentStrategy.RECREATE
+
+        if proto is isolate_proto.DeploymentStrategy.RECREATE:
+            return DeploymentStrategy.RECREATE
+        elif proto is isolate_proto.DeploymentStrategy.ROLLING:
+            return DeploymentStrategy.ROLLING
+        else:
+            raise ValueError(f"Unknown DeploymentStrategy: {proto}")
+
+    def to_proto(self) -> isolate_proto.DeploymentStrategy.ValueType:
+        if self is DeploymentStrategy.RECREATE:
+            return isolate_proto.DeploymentStrategy.RECREATE
+        elif self is DeploymentStrategy.ROLLING:
+            return isolate_proto.DeploymentStrategy.ROLLING
+        else:
+            raise ValueError(f"Unknown DeploymentStrategy: {self}")
+
+
 @from_grpc.register(isolate_proto.ApplicationInfo)
 def _from_grpc_application_info(
-    message: isolate_proto.ApplicationInfo
+    message: isolate_proto.ApplicationInfo,
 ) -> ApplicationInfo:
     return ApplicationInfo(
         application_id=message.application_id,
@@ -285,6 +367,12 @@ def _from_grpc_application_info(
         max_multiplexing=message.max_multiplexing,
         active_runners=message.active_runners,
         min_concurrency=message.min_concurrency,
+        concurrency_buffer=message.concurrency_buffer,
+        machine_types=list(message.machine_types),
+        request_timeout=message.request_timeout,
+        startup_timeout=message.startup_timeout,
+        valid_regions=list(message.valid_regions),
+        created_at=isolate_proto.datetime_from_timestamp(message.created_at),
     )
 
 
@@ -308,16 +396,31 @@ def _from_grpc_alias_info(message: isolate_proto.AliasInfo) -> AliasInfo:
         max_multiplexing=message.max_multiplexing,
         active_runners=message.active_runners,
         min_concurrency=message.min_concurrency,
+        concurrency_buffer=message.concurrency_buffer,
+        machine_types=list(message.machine_types),
+        request_timeout=message.request_timeout,
+        startup_timeout=message.startup_timeout,
+        valid_regions=list(message.valid_regions),
     )
 
 
 @from_grpc.register(isolate_proto.RunnerInfo)
 def _from_grpc_runner_info(message: isolate_proto.RunnerInfo) -> RunnerInfo:
+    from isolate.server import definitions as worker_definitions
+
+    external_metadata = worker_definitions.struct_to_dict(message.external_metadata)
     return RunnerInfo(
         runner_id=message.runner_id,
         in_flight_requests=message.in_flight_requests,
-        expiration_countdown=message.expiration_countdown,
+        expiration_countdown=(
+            message.expiration_countdown
+            if message.HasField("expiration_countdown")
+            else None
+        ),
         uptime=timedelta(seconds=message.uptime),
+        external_metadata=external_metadata,
+        revision=message.revision,
+        alias=message.alias,
     )
 
 
@@ -361,7 +464,8 @@ def _from_grpc_hosted_run_result(
 
 @dataclass
 class MachineRequirements:
-    machine_type: str
+    machine_types: list[str]
+    num_gpus: int | None = field(default=None)
     keep_alive: int = FAL_SERVERLESS_DEFAULT_KEEP_ALIVE
     base_image: str | None = None
     exposed_port: int | None = None
@@ -370,6 +474,19 @@ class MachineRequirements:
     max_concurrency: int | None = None
     max_multiplexing: int | None = None
     min_concurrency: int | None = None
+    concurrency_buffer: int | None = None
+    request_timeout: int | None = None
+    startup_timeout: int | None = None
+
+    def __post_init__(self):
+        if isinstance(self.machine_types, str):
+            self.machine_types = [self.machine_types]
+
+        if not isinstance(self.machine_types, list):
+            raise ValueError("machine_types must be a list of strings.")
+
+        if not self.machine_types:
+            raise ValueError("No machine type provided.")
 
 
 @dataclass
@@ -451,16 +568,22 @@ class FalServerlessConnection:
         function: Callable[..., ResultT],
         environments: list[isolate_proto.EnvironmentDefinition],
         application_name: str | None = None,
-        application_auth_mode: Literal["public", "private", "shared"] | None = None,
+        auth_mode: AuthMode = None,
         *,
         serialization_method: str = _DEFAULT_SERIALIZATION_METHOD,
         machine_requirements: MachineRequirements | None = None,
         metadata: dict[str, Any] | None = None,
+        deployment_strategy: Literal["recreate", "rolling"] = "recreate",
+        scale: bool = True,
+        private_logs: bool = False,
     ) -> Iterator[isolate_proto.RegisterApplicationResult]:
         wrapped_function = to_serialized_object(function, serialization_method)
         if machine_requirements:
             wrapped_requirements = isolate_proto.MachineRequirements(
-                machine_type=machine_requirements.machine_type,
+                # NOTE: backwards compatibility with old API
+                machine_type=machine_requirements.machine_types[0],
+                machine_types=machine_requirements.machine_types,
+                num_gpus=machine_requirements.num_gpus,
                 keep_alive=machine_requirements.keep_alive,
                 base_image=machine_requirements.base_image,
                 exposed_port=machine_requirements.exposed_port,
@@ -470,30 +593,42 @@ class FalServerlessConnection:
                 ),
                 max_concurrency=machine_requirements.max_concurrency,
                 min_concurrency=machine_requirements.min_concurrency,
+                concurrency_buffer=machine_requirements.concurrency_buffer,
                 max_multiplexing=machine_requirements.max_multiplexing,
+                request_timeout=machine_requirements.request_timeout,
+                startup_timeout=machine_requirements.startup_timeout,
             )
         else:
             wrapped_requirements = None
 
-        if application_auth_mode == "public":
-            auth_mode = isolate_proto.ApplicationAuthMode.PUBLIC
-        elif application_auth_mode == "shared":
-            auth_mode = isolate_proto.ApplicationAuthMode.SHARED
+        if auth_mode == "public":
+            auth = isolate_proto.ApplicationAuthMode.PUBLIC
+        elif auth_mode == "shared":
+            auth = isolate_proto.ApplicationAuthMode.SHARED
+        elif auth_mode == "private":
+            auth = isolate_proto.ApplicationAuthMode.PRIVATE
         else:
-            auth_mode = isolate_proto.ApplicationAuthMode.PRIVATE
+            auth = None
 
         struct_metadata = None
         if metadata:
             struct_metadata = isolate_proto.Struct()
             struct_metadata.update(metadata)
 
+        deployment_strategy_proto = DeploymentStrategy[
+            deployment_strategy.upper()
+        ].to_proto()
+
         request = isolate_proto.RegisterApplicationRequest(
             function=wrapped_function,
             environments=environments,
             machine_requirements=wrapped_requirements,
             application_name=application_name,
-            auth_mode=auth_mode,
+            auth_mode=auth,
             metadata=struct_metadata,
+            deployment_strategy=deployment_strategy_proto,
+            scale=scale,
+            private_logs=private_logs,
         )
         for partial_result in self.stub.RegisterApplication(request):
             yield from_grpc(partial_result)
@@ -508,6 +643,11 @@ class FalServerlessConnection:
         max_multiplexing: int | None = None,
         max_concurrency: int | None = None,
         min_concurrency: int | None = None,
+        concurrency_buffer: int | None = None,
+        request_timeout: int | None = None,
+        startup_timeout: int | None = None,
+        valid_regions: list[str] | None = None,
+        machine_types: list[str] | None = None,
     ) -> AliasInfo:
         request = isolate_proto.UpdateApplicationRequest(
             application_name=application_name,
@@ -515,14 +655,23 @@ class FalServerlessConnection:
             max_multiplexing=max_multiplexing,
             max_concurrency=max_concurrency,
             min_concurrency=min_concurrency,
+            concurrency_buffer=concurrency_buffer,
+            request_timeout=request_timeout,
+            startup_timeout=startup_timeout,
+            valid_regions=valid_regions,
+            machine_types=machine_types,
         )
         res: isolate_proto.UpdateApplicationResult = self.stub.UpdateApplication(
             request
         )
         return from_grpc(res.alias_info)
 
-    def list_applications(self) -> list[ApplicationInfo]:
-        request = isolate_proto.ListApplicationsRequest()
+    def list_applications(
+        self, application_name: str | None = None
+    ) -> list[ApplicationInfo]:
+        request = isolate_proto.ListApplicationsRequest(
+            application_name=application_name
+        )
         res: isolate_proto.ListApplicationsResult = self.stub.ListApplications(request)
         return [from_grpc(app) for app in res.applications]
 
@@ -545,7 +694,10 @@ class FalServerlessConnection:
         wrapped_function = to_serialized_object(function, serialization_method)
         if machine_requirements:
             wrapped_requirements = isolate_proto.MachineRequirements(
-                machine_type=machine_requirements.machine_type,
+                # NOTE: backwards compatibility with old API
+                machine_type=machine_requirements.machine_types[0],
+                machine_types=machine_requirements.machine_types,
+                num_gpus=machine_requirements.num_gpus,
                 keep_alive=machine_requirements.keep_alive,
                 base_image=machine_requirements.base_image,
                 exposed_port=machine_requirements.exposed_port,
@@ -556,6 +708,9 @@ class FalServerlessConnection:
                 max_concurrency=machine_requirements.max_concurrency,
                 max_multiplexing=machine_requirements.max_multiplexing,
                 min_concurrency=machine_requirements.min_concurrency,
+                concurrency_buffer=machine_requirements.concurrency_buffer,
+                request_timeout=machine_requirements.request_timeout,
+                startup_timeout=machine_requirements.startup_timeout,
             )
         else:
             wrapped_requirements = None
@@ -569,33 +724,44 @@ class FalServerlessConnection:
             request.setup_func.MergeFrom(
                 to_serialized_object(setup_function, serialization_method)
             )
-        for partial_result in self.stub.Run(request):
-            yield from_grpc(partial_result)
+        stream = self.stub.Run(request)
+        for partial_result in stream:
+            res = from_grpc(partial_result)
+            res.stream = stream
+            yield res
 
     def create_alias(
         self,
         alias: str,
         revision: str,
-        auth_mode: Literal["public", "private", "shared"],
-    ):
+        auth_mode: AuthMode,
+    ) -> AliasInfo:
         if auth_mode == "public":
             auth = isolate_proto.ApplicationAuthMode.PUBLIC
         elif auth_mode == "shared":
             auth = isolate_proto.ApplicationAuthMode.SHARED
-        else:
+        elif auth_mode == "private":
             auth = isolate_proto.ApplicationAuthMode.PRIVATE
+        else:
+            auth = None
 
         request = isolate_proto.SetAliasRequest(
             alias=alias,
             revision=revision,
             auth_mode=auth,
         )
-        self.stub.SetAlias(request)
+        res = self.stub.SetAlias(request)
+        return from_grpc(res.alias_info)
 
-    def delete_alias(self, alias: str) -> str:
+    def delete_alias(self, alias: str) -> str | None:
         request = isolate_proto.DeleteAliasRequest(alias=alias)
-        res: isolate_proto.DeleteAliasResult = self.stub.DeleteAlias(request)
-        return res.revision
+        try:
+            res: isolate_proto.DeleteAliasResult = self.stub.DeleteAlias(request)
+            return res.revision
+        except grpc.RpcError as e:
+            if e.code() == grpc.StatusCode.NOT_FOUND:
+                return None
+            raise
 
     def list_aliases(self) -> list[AliasInfo]:
         request = isolate_proto.ListAliasesRequest()
@@ -625,3 +791,12 @@ class FalServerlessConnection:
             )
             for secret in response.secrets
         ]
+
+    def kill_runner(self, runner_id: str) -> None:
+        request = isolate_proto.KillRunnerRequest(runner_id=runner_id)
+        self.stub.KillRunner(request)
+
+    def list_runners(self) -> list[RunnerInfo]:
+        request = isolate_proto.ListRunnersRequest()
+        response = self.stub.ListRunners(request)
+        return [from_grpc(runner) for runner in response.runners]
